@@ -1,3 +1,6 @@
+// SPDX-FileCopyrightText: 2020 Carl Schwan <carlschwan@kde.org>
+// SPDX-License-Identifier: GPL-3.0-or-later
+
 #include "chatdocumenthandler.h"
 
 #include <QQmlFile>
@@ -146,7 +149,7 @@ QVariantMap ChatDocumentHandler::getAutocompletionInfo()
         };
     }
 
-    if (autoCompletePrefix.startsWith("@") || autoCompletePrefix.startsWith(":")) {
+    if (autoCompletePrefix.startsWith("@") || autoCompletePrefix.startsWith(":") || autoCompletePrefix.startsWith("/")) {
         m_autoCompleteBeginPosition = textBeforeCursor.lastIndexOf(" ") + 1; // 1 == space
 
         if (autoCompletePrefix.startsWith("@")) {
@@ -156,6 +159,14 @@ QVariantMap ChatDocumentHandler::getAutocompletionInfo()
                 {"type", AutoCompletionType::User},
             };
         }
+
+        if (autoCompletePrefix.startsWith("/")) {
+            return QVariantMap{
+                {"keyword", autoCompletePrefix},
+                {"type", AutoCompletionType::Command},
+            };
+        }
+
         return QVariantMap{
             {"keyword", autoCompletePrefix},
             {"type", AutoCompletionType::Emoji},
