@@ -1,40 +1,42 @@
-/**
- * SPDX-FileCopyrightText: 2018-2019 Black Hat <bhat@encom.eu.org>
- * SPDX-FileCopyrightText: 2020 Tobias Fella <fella@posteo.de>
- *
- * SPDX-License-Identifier: GPL-3.0-only
- */
+// SPDX-FileCopyrightText: 2018-2019 Black Hat <bhat@encom.eu.org>
+// SPDX-FileCopyrightText: 2020 Tobias Fella <fella@posteo.de>
+// SPDX-License-Identifier: GPL-3.0-only
+
 #include "controller.h"
 
-#ifndef Q_OS_ANDROID
 #include <qt5keychain/keychain.h>
-#endif
+
 #include <KConfig>
 #include <KConfigGroup>
-#include <KWindowConfig>
 #include <KLocalizedString>
+#include <KWindowConfig>
 
+#include <QAuthenticator>
 #include <QClipboard>
+#include <QCloseEvent>
 #include <QDebug>
-#include <QQuickWindow>
+#include <QDesktopServices>
 #include <QDir>
 #include <QElapsedTimer>
 #include <QFile>
 #include <QFileInfo>
+#include <QGuiApplication>
+#include <QMovie>
+#include <QNetworkConfigurationManager>
+#include <QNetworkReply>
+#include <QPixmap>
+#include <QQuickWindow>
 #include <QStandardPaths>
+#include <QStringBuilder>
 #include <QSysInfo>
 #include <QTimer>
-#include <QCloseEvent>
-#include <QDesktopServices>
-#include <QMovie>
-#include <QPixmap>
-#include <QAuthenticator>
-#include <QNetworkReply>
-#include <QStringBuilder>
 #include <utility>
 
+#include <signal.h>
+
 #include "csapi/account-data.h"
 #include "csapi/content-repo.h"
+#include "csapi/joining.h"
 #include "csapi/logout.h"
 #include "csapi/profile.h"
 #include "csapi/registration.h"
@@ -46,40 +48,75 @@
 #include "neochatuser.h"
 #include "settings.h"
 #include "utils.h"
+#include "roommanager.h"
 #include <KStandardShortcut>
 
 #ifndef Q_OS_ANDROID
 #include "trayicon.h"
 #endif
 
+using namespace Quotient;
+
 Controller::Controller(QObject *parent)
     : QObject(parent)
+    , m_mgr(new QNetworkConfigurationManager(this))
 {
     Connection::setRoomType<NeoChatRoom>();
     Connection::setUserType<NeoChatUser>();
 
 #ifndef Q_OS_ANDROID
     TrayIcon *trayIcon = new TrayIcon(this);
-    if(NeoChatConfig::self()->systemTray()) {
+    if (NeoChatConfig::self()->systemTray()) {
         trayIcon->show();
         connect(trayIcon, &TrayIcon::showWindow, this, &Controller::showWindow);
-        QApplication::setQuitOnLastWindowClosed(false);
+        QGuiApplication::setQuitOnLastWindowClosed(false);
     }
-    connect(NeoChatConfig::self(), &NeoChatConfig::SystemTrayChanged, this, [=](){
-        if(NeoChatConfig::self()->systemTray()) {
+    connect(NeoChatConfig::self(), &NeoChatConfig::SystemTrayChanged, this, [=]() {
+        if (NeoChatConfig::self()->systemTray()) {
             trayIcon->show();
             connect(trayIcon, &TrayIcon::showWindow, this, &Controller::showWindow);
         } else {
             trayIcon->hide();
             disconnect(trayIcon, &TrayIcon::showWindow, this, &Controller::showWindow);
         }
-        QApplication::setQuitOnLastWindowClosed(!NeoChatConfig::self()->systemTray());
+        QGuiApplication::setQuitOnLastWindowClosed(!NeoChatConfig::self()->systemTray());
     });
 #endif
 
     QTimer::singleShot(0, this, [=] {
         invokeLogin();
     });
+
+    QObject::connect(QGuiApplication::instance(), &QCoreApplication::aboutToQuit, QGuiApplication::instance(), [] {
+        NeoChatConfig::self()->save();
+    });
+
+#ifndef Q_OS_WINDOWS
+    // Setup Unix signal handlers
+    const auto unixExitHandler = [](int /*sig*/) -> void {
+        QCoreApplication::quit();
+    };
+
+    const int quitSignals[] = {SIGQUIT, SIGINT, SIGTERM, SIGHUP};
+
+    sigset_t blockingMask;
+    sigemptyset(&blockingMask);
+    for (const auto sig : quitSignals) {
+        sigaddset(&blockingMask, sig);
+    }
+
+    struct sigaction sa;
+    sa.sa_handler = unixExitHandler;
+    sa.sa_mask = blockingMask;
+    sa.sa_flags = 0;
+
+    for (auto sig : quitSignals) {
+        sigaction(sig, &sa, nullptr);
+    }
+#endif
+
+    connect(m_mgr, &QNetworkConfigurationManager::onlineStateChanged,
+            this, &Controller::isOnlineChanged);
 }
 
 Controller::~Controller()
@@ -130,7 +167,7 @@ void Controller::loginWithAccessToken(const QString &serverAddr, const QString &
         setActiveConnection(conn);
     });
     connect(conn, &Connection::networkError, this, [=](QString error, const QString &, int, int) {
-        Q_EMIT errorOccured("Network Error", std::move(error));
+        Q_EMIT errorOccured(i18n("Network Error: %1", error));
     });
     conn->connectWithToken(user, token, deviceName);
 }
@@ -145,7 +182,6 @@ void Controller::logout(Connection *conn, bool serverSideLogout)
     SettingsGroup("Accounts").remove(conn->userId());
     QFile(accessTokenFileName(AccountSettings(conn->userId()))).remove();
 
-#ifndef Q_OS_ANDROID
     QKeychain::DeletePasswordJob job(qAppName());
     job.setAutoDelete(true);
     job.setKey(conn->userId());
@@ -153,11 +189,7 @@ void Controller::logout(Connection *conn, bool serverSideLogout)
     QKeychain::DeletePasswordJob::connect(&job, &QKeychain::Job::finished, &loop, &QEventLoop::quit);
     job.start();
     loop.exec();
-#else
-    KConfig config("neochat_tokens");
-    KConfigGroup tokensGroup(&config, "Tokens");
-    tokensGroup.deleteEntry(conn->userId());
-#endif
+
     conn->stopSync();
     Q_EMIT conn->stateChanged();
     Q_EMIT conn->loggedOut();
@@ -171,7 +203,7 @@ void Controller::logout(Connection *conn, bool serverSideLogout)
     }
     auto logoutJob = conn->callApi<LogoutJob>();
     connect(logoutJob, &LogoutJob::failure, this, [=] {
-        Q_EMIT errorOccured("Server-side Logout Failed", logoutJob->errorString());
+        Q_EMIT errorOccured(i18n("Server-side Logout Failed: %1", logoutJob->errorString()));
     });
 }
 
@@ -195,8 +227,8 @@ void Controller::addConnection(Connection *c)
         dropConnection(c);
     });
 
-    connect(c, &Connection::requestFailed, this, [=] (BaseJob *job) {
-        if(job->error() == BaseJob::UserConsentRequiredError) {
+    connect(c, &Connection::requestFailed, this, [=](BaseJob *job) {
+        if (job->error() == BaseJob::UserConsentRequiredError) {
             Q_EMIT userConsentRequired(job->errorUrl());
         }
     });
@@ -243,16 +275,16 @@ void Controller::invokeLogin()
             });
             connect(connection, &Connection::loginError, this, [=](const QString &error, const QString &) {
                 if (error == "Unrecognised access token") {
-                    Q_EMIT errorOccured(i18n("Login Failed"), i18n("Access Token invalid or revoked"));
+                    Q_EMIT errorOccured(i18n("Login Failed: Access Token invalid or revoked"));
                     logout(connection, false);
                 } else {
-                    Q_EMIT errorOccured(i18n("Login Failed"), error);
+                    Q_EMIT errorOccured(i18n("Login Failed", error));
                     logout(connection, true);
                 }
                 Q_EMIT initiated();
             });
             connect(connection, &Connection::networkError, this, [=](const QString &error, const QString &, int, int) {
-                Q_EMIT errorOccured("Network Error", error);
+                Q_EMIT errorOccured(i18n("Network Error: %1", error));
             });
             connection->connectWithToken(account.userId(), accessToken, account.deviceId());
         }
@@ -279,7 +311,6 @@ QByteArray Controller::loadAccessTokenFromFile(const AccountSettings &account)
 
 QByteArray Controller::loadAccessTokenFromKeyChain(const AccountSettings &account)
 {
-#ifndef Q_OS_ANDROID
     qDebug() << "Read the access token from the keychain for " << account.userId();
     QKeychain::ReadPasswordJob job(qAppName());
     job.setAutoDelete(false);
@@ -313,13 +344,6 @@ QByteArray Controller::loadAccessTokenFromKeyChain(const AccountSettings &accoun
     }
 
     return accessToken;
-#else
-    qDebug() << "Saving access token to KConfig";
-    KConfig config("neochat_tokens");
-    KConfigGroup tokensGroup(&config, "Tokens");
-    QString token = tokensGroup.readEntry(account.userId(), QString());
-    return token.toLatin1();
-#endif
 }
 
 bool Controller::saveAccessTokenToFile(const AccountSettings &account, const QByteArray &accessToken)
@@ -330,7 +354,7 @@ bool Controller::saveAccessTokenToFile(const AccountSettings &account, const QBy
 
     auto fileDir = QFileInfo(accountTokenFile).dir();
     if (!((fileDir.exists() || fileDir.mkpath(".")) && accountTokenFile.open(QFile::WriteOnly))) {
-        Q_EMIT errorOccured("I/O Denied", "Cannot save access token.");
+        Q_EMIT errorOccured("I/O Denied: Cannot save access token.");
     } else {
         accountTokenFile.write(accessToken);
         return true;
@@ -340,7 +364,6 @@ bool Controller::saveAccessTokenToFile(const AccountSettings &account, const QBy
 
 bool Controller::saveAccessTokenToKeyChain(const AccountSettings &account, const QByteArray &accessToken)
 {
-#ifndef Q_OS_ANDROID
     qDebug() << "Save the access token to the keychain for " << account.userId();
     QKeychain::WritePasswordJob job(qAppName());
     job.setAutoDelete(false);
@@ -355,16 +378,9 @@ bool Controller::saveAccessTokenToKeyChain(const AccountSettings &account, const
         qWarning() << "Could not save access token to the keychain: " << qPrintable(job.errorString());
         return saveAccessTokenToFile(account, accessToken);
     }
-
-#else
-    KConfig config("neochat_tokens");
-    KConfigGroup tokensGroup(&config, "Tokens");
-    tokensGroup.writeEntry(account.userId(), accessToken);
-#endif
     return true;
 }
 
-
 void Controller::playAudio(const QUrl &localFile)
 {
     auto player = new QMediaPlayer;
@@ -378,7 +394,11 @@ void Controller::playAudio(const QUrl &localFile)
 void Controller::changeAvatar(Connection *conn, const QUrl &localFile)
 {
     auto job = conn->uploadFile(localFile.toLocalFile());
+#ifdef QUOTIENT_07
+    if(isJobPending(job)) {
+#else
     if (isJobRunning(job)) {
+#endif
         connect(job, &BaseJob::success, this, [conn, job] {
             conn->callApi<SetAvatarUrlJob>(conn->userId(), job->contentUri());
         });
@@ -404,6 +424,16 @@ KAboutData Controller::aboutData() const
     return m_aboutData;
 }
 
+bool Controller::supportSystemTray() const
+{
+#ifdef Q_OS_ANDROID
+    return false;
+#else
+    QString de = getenv("XDG_CURRENT_DESKTOP");
+    return de != QStringLiteral("GNOME") && de != QStringLiteral("Pantheon");
+#endif
+}
+
 void Controller::changePassword(Connection *connection, const QString &currentPassword, const QString &newPassword)
 {
     NeochatChangePasswordJob *job = connection->callApi<NeochatChangePasswordJob>(newPassword, false);
@@ -432,6 +462,21 @@ void Controller::changePassword(Connection *connection, const QString &currentPa
     });
 }
 
+bool Controller::setAvatar(Connection *connection, const QUrl &avatarSource)
+{
+    User *localUser = connection->user();
+    QString decoded = avatarSource.path();
+    if (decoded.isEmpty()) {
+        connection->callApi<SetAvatarUrlJob>(localUser->id(), "");
+        return true;
+    }
+    if (QImageReader(decoded).read().isNull()) {
+        return false;
+    } else {
+        return localUser->setAvatar(decoded);
+    }
+}
+
 NeochatChangePasswordJob::NeochatChangePasswordJob(const QString &newPassword, bool logoutDevices, const Omittable<QJsonObject> &auth)
     : BaseJob(HttpVerb::Post, QStringLiteral("ChangePasswordJob"), QStringLiteral("/_matrix/client/r0") % "/account/password")
 {
@@ -454,13 +499,13 @@ int Controller::accountCount() const
 
 bool Controller::quitOnLastWindowClosed()
 {
-    return QApplication::quitOnLastWindowClosed();
+    return QGuiApplication::quitOnLastWindowClosed();
 }
 
 void Controller::setQuitOnLastWindowClosed(bool value)
 {
     if (quitOnLastWindowClosed() != value) {
-        QApplication::setQuitOnLastWindowClosed(value);
+        QGuiApplication::setQuitOnLastWindowClosed(value);
         Q_EMIT quitOnLastWindowClosedChanged();
     }
 }
@@ -511,12 +556,62 @@ void Controller::saveWindowGeometry(QQuickWindow *window)
     dataResource.sync();
 }
 
-
 NeochatDeleteDeviceJob::NeochatDeleteDeviceJob(const QString &deviceId, const Omittable<QJsonObject> &auth)
     : Quotient::BaseJob(HttpVerb::Delete, QStringLiteral("DeleteDeviceJob"), QStringLiteral("/_matrix/client/r0/devices/%1").arg(deviceId))
 {
     QJsonObject _data;
     addParam<IfNotEmpty>(_data, QStringLiteral("auth"), auth);
     setRequestData(std::move(_data));
+}
 
+void Controller::createRoom(const QString &name, const QString &topic)
+{
+    auto createRoomJob = m_connection->createRoom(Connection::PublishRoom, "", name, topic, QStringList());
+    Quotient::CreateRoomJob::connect(createRoomJob, &CreateRoomJob::failure, [=] {
+        Q_EMIT errorOccured(i18n("Room creation failed: \"%1\"", createRoomJob->errorString()));
+    });
+}
+
+bool Controller::isOnline() const
+{
+    return m_mgr->isOnline();
+}
+
+void Controller::joinRoom(const QString &alias)
+{
+    if (!alias.contains(":")) {
+        Q_EMIT errorOccured(i18n("The room id you are trying to join is not valid"));
+        return;
+    }
+
+    const auto knownServer = alias.mid(alias.indexOf(":") + 1);
+    auto joinRoomJob = m_connection->joinRoom(alias, QStringList{knownServer});
+
+    connect(joinRoomJob, &JoinRoomJob::failure, [=] {
+        Q_EMIT errorOccured(i18n("Server error when joining the room \"%1\": %2", joinRoomJob->errorString()));
+    });
+    connect(joinRoomJob, &JoinRoomJob::success, [this, joinRoomJob] {
+        Q_EMIT errorOccured(joinRoomJob->roomId());
+    });
+}
+
+void Controller::openOrCreateDirectChat(NeoChatUser *user)
+{
+    const auto existing = activeConnection()->directChats();
+
+    if (existing.contains(user)) {
+        RoomManager::instance().enterRoom(static_cast<NeoChatRoom *>(activeConnection()->room(existing.value(user))));
+        return;
+    }
+    activeConnection()->requestDirectChat(user);
+}
+
+QString Controller::formatByteSize(double size, int precision) const
+{
+    return KFormat().formatByteSize(size, precision);
+}
+
+QString Controller::formatDuration(quint64 msecs, KFormat::DurationFormatOptions options) const
+{
+    return KFormat().formatDuration(msecs, options);
 }
