@@ -1,18 +1,16 @@
-/**
- * SPDX-FileCopyrightText: 2018-2019 Black Hat <bhat@encom.eu.org>
- *
- * SPDX-License-Identifier: GPL-3.0-only
- */
+// SPDX-FileCopyrightText: 2018-2019 Black Hat <bhat@encom.eu.org>
+// SPDX-License-Identifier: GPL-3.0-only
 
 #pragma once
 
-#include <QApplication>
 #include <QMediaPlayer>
-#include <QMenu>
 #include <QObject>
 
 #include <KAboutData>
+#include <KFormat>
+
 class QKeySequences;
+class QNetworkConfigurationManager;
 
 #include "connection.h"
 #include "csapi/list_public_rooms.h"
@@ -21,6 +19,7 @@ class QKeySequences;
 #include "user.h"
 
 class NeoChatRoom;
+class NeoChatUser;
 class QQuickWindow;
 
 using namespace Quotient;
@@ -33,6 +32,8 @@ class Controller : public QObject
     Q_PROPERTY(Connection *activeConnection READ activeConnection WRITE setActiveConnection NOTIFY activeConnectionChanged)
     Q_PROPERTY(bool busy READ busy WRITE setBusy NOTIFY busyChanged)
     Q_PROPERTY(KAboutData aboutData READ aboutData WRITE setAboutData NOTIFY aboutDataChanged)
+    Q_PROPERTY(bool supportSystemTray READ supportSystemTray CONSTANT)
+    Q_PROPERTY(bool isOnline READ isOnline NOTIFY isOnlineChanged)
 
 public:
     static Controller &instance();
@@ -49,6 +50,8 @@ public:
 
     Q_INVOKABLE void changePassword(Quotient::Connection *connection, const QString &currentPassword, const QString &newPassword);
 
+    Q_INVOKABLE bool setAvatar(Quotient::Connection *connection, const QUrl &avatarSource);
+
     [[nodiscard]] int accountCount() const;
 
     [[nodiscard]] static bool quitOnLastWindowClosed();
@@ -60,6 +63,8 @@ public:
     void setAboutData(const KAboutData &aboutData);
     [[nodiscard]] KAboutData aboutData() const;
 
+    [[nodiscard]] bool supportSystemTray() const;
+
     bool saveAccessTokenToFile(const AccountSettings &account, const QByteArray &accessToken);
     bool saveAccessTokenToKeyChain(const AccountSettings &account, const QByteArray &accessToken);
 
@@ -70,6 +75,18 @@ public:
     };
     Q_ENUM(PasswordStatus);
 
+    /// \brief Create new room for a group chat.
+    Q_INVOKABLE void createRoom(const QString &name, const QString &topic);
+
+    /// \brief Join a room.
+    Q_INVOKABLE void joinRoom(const QString &alias);
+
+    bool isOnline() const;
+
+    Q_INVOKABLE QString formatDuration(quint64 msecs, KFormat::DurationFormatOptions options = KFormat::DefaultDuration) const;
+    Q_INVOKABLE QString formatByteSize(double size, int precision = 1) const;
+
+    Q_INVOKABLE void openOrCreateDirectChat(NeoChatUser *user);
 private:
     explicit Controller(QObject *parent = nullptr);
     ~Controller() override;
@@ -92,7 +109,7 @@ private Q_SLOTS:
 Q_SIGNALS:
     void busyChanged();
     /// Error occurred because of user inputs
-    void errorOccured(QString error, QString detail);
+    void errorOccured(const QString &error);
 
     /// Error occurred because of server or bug in NeoChat
     void globalErrorOccured(QString error, QString detail);
@@ -108,9 +125,9 @@ Q_SIGNALS:
     void aboutDataChanged();
     void passwordStatus(Controller::PasswordStatus _t1);
     void showWindow();
-    void openRoom(NeoChatRoom *room);
     void userConsentRequired(QUrl url);
     void testConnectionResult(const QString &connection, bool usable);
+    void isOnlineChanged(bool isOnline);
 
 public Q_SLOTS:
     void logout(Quotient::Connection *conn, bool serverSideLogout);
@@ -118,6 +135,9 @@ public Q_SLOTS:
     void changeAvatar(Quotient::Connection *conn, const QUrl &localFile);
     static void markAllMessagesAsRead(Quotient::Connection *conn);
     void saveWindowGeometry(QQuickWindow *);
+
+private:
+    QNetworkConfigurationManager *m_mgr;
 };
 
 // TODO libQuotient 0.7: Drop
