@@ -1,8 +1,6 @@
-/**
- * SPDX-FileCopyrightText: 2018-2019 Black Hat <bhat@encom.eu.org>
- *
- * SPDX-License-Identifier: GPL-3.0-only
- */
+// SPDX-FileCopyrightText: 2018-2019 Black Hat <bhat@encom.eu.org>
+// SPDX-License-Identifier: GPL-3.0-only
+
 #include <QCommandLineParser>
 #include <QFontDatabase>
 #include <QGuiApplication>
@@ -12,6 +10,13 @@
 #include <QQmlContext>
 #include <QQuickStyle>
 #include <QQuickWindow>
+#include <QDebug>
+
+#ifdef Q_OS_ANDROID
+#include <QGuiApplication>
+#else
+#include <QApplication>
+#endif
 
 #include <KAboutData>
 #ifdef HAVE_KDBUSADDONS
@@ -24,8 +29,11 @@
 #include "neochat-version.h"
 
 #include "accountlistmodel.h"
+#include "actionshandler.h"
+#include "chatboxhelper.h"
 #include "chatdocumenthandler.h"
 #include "clipboard.h"
+#include "commandmodel.h"
 #include "controller.h"
 #include "csapi/joining.h"
 #include "csapi/leaving.h"
@@ -41,12 +49,12 @@
 #include "neochatuser.h"
 #include "notificationsmanager.h"
 #include "publicroomlistmodel.h"
-#include "room.h"
+#include <room.h>
 #include "roomlistmodel.h"
+#include "roommanager.h"
 #include "sortfilterroomlistmodel.h"
 #include "userdirectorylistmodel.h"
 #include "userlistmodel.h"
-#include "actionshandler.h"
 
 using namespace Quotient;
 
@@ -55,7 +63,7 @@ Q_DECL_EXPORT
 #endif
 int main(int argc, char *argv[])
 {
-    QApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
+    QGuiApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
 
     QNetworkProxyFactory::setUseSystemConfiguration(true);
 
@@ -71,22 +79,46 @@ int main(int argc, char *argv[])
 #endif
 
 #ifdef Q_OS_WINDOWS
+    if (AttachConsole(ATTACH_PARENT_PROCESS)) {
+        freopen("CONOUT$", "w", stdout);
+        freopen("CONOUT$", "w", stderr);
+    }
+
     QApplication::setStyle(QStringLiteral("breeze"));
+    auto font = app.font();
+    font.setPointSize(10);
+    app.setFont(font);
 #endif
 
-    QApplication::setOrganizationName("KDE");
+    QGuiApplication::setOrganizationName("KDE");
 
-    KAboutData about(QStringLiteral("neochat"), i18n("Neochat"), QStringLiteral(NEOCHAT_VERSION_STRING), i18n("Matrix client"), KAboutLicense::GPL_V3, i18n("© 2018-2020 Black Hat, 2020 KDE Community"));
+    KAboutData about(QStringLiteral("neochat"),
+                     i18n("NeoChat"),
+                     QStringLiteral(NEOCHAT_VERSION_STRING),
+                     i18n("Matrix client"),
+                     KAboutLicense::GPL_V3,
+                     i18n("© 2018-2020 Black Hat, 2020 KDE Community"));
     about.addAuthor(i18n("Black Hat"), QString(), QStringLiteral("bhat@encom.eu.org"));
     about.addAuthor(i18n("Carl Schwan"), QString(), QStringLiteral("carl@carlschwan.eu"));
     about.addAuthor(i18n("Tobias Fella"), QString(), QStringLiteral("fella@posteo.de"));
     about.setOrganizationDomain("kde.org");
 
     KAboutData::setApplicationData(about);
-    QApplication::setWindowIcon(QIcon::fromTheme(QStringLiteral("org.kde.neochat")));
+    QGuiApplication::setWindowIcon(QIcon::fromTheme(QStringLiteral("org.kde.neochat")));
 
 #ifdef HAVE_KDBUSADDONS
     KDBusService service(KDBusService::Unique);
+    service.connect(&service,
+                    &KDBusService::activateRequested,
+                    &RoomManager::instance(),
+                    [](const QStringList &arguments, const QString &workingDirectory) {
+                        Q_UNUSED(workingDirectory);
+                        auto args = arguments;
+                        args.removeFirst();
+                        for (const auto &arg : args) {
+                            RoomManager::instance().openResource(arg);
+                        }
+                    });
 #endif
 
 #ifdef NEOCHAT_FLATPAK
@@ -100,12 +132,15 @@ int main(int argc, char *argv[])
     FileTypeSingleton fileTypeSingleton;
 
     Login *login = new Login();
+    ChatBoxHelper chatBoxHelper;
 
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "Controller", &Controller::instance());
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "Clipboard", &clipboard);
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "Config", config);
+    qmlRegisterSingletonInstance<RoomManager>("org.kde.neochat", 1, 0, "RoomManager", &RoomManager::instance());
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "FileType", &fileTypeSingleton);
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "LoginHelper", login);
+    qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "ChatBoxHelper", &chatBoxHelper);
     qmlRegisterType<AccountListModel>("org.kde.neochat", 1, 0, "AccountListModel");
     qmlRegisterType<ActionsHandler>("org.kde.neochat", 1, 0, "ActionsHandler");
     qmlRegisterType<ChatDocumentHandler>("org.kde.neochat", 1, 0, "ChatDocumentHandler");
@@ -115,7 +150,16 @@ int main(int argc, char *argv[])
     qmlRegisterType<MessageFilterModel>("org.kde.neochat", 1, 0, "MessageFilterModel");
     qmlRegisterType<PublicRoomListModel>("org.kde.neochat", 1, 0, "PublicRoomListModel");
     qmlRegisterType<UserDirectoryListModel>("org.kde.neochat", 1, 0, "UserDirectoryListModel");
-    qmlRegisterType<EmojiModel>("org.kde.neochat", 1, 0, "EmojiModel");
+    qmlRegisterSingletonType<EmojiModel>("org.kde.neochat", 1, 0, "EmojiModel", [](QQmlEngine *engine2, QJSEngine *scriptEngine) -> QObject * {
+        Q_UNUSED(scriptEngine);
+        Q_UNUSED(engine2);
+        return new EmojiModel();
+    });
+    qmlRegisterSingletonType<CommandModel>("org.kde.neochat", 1, 0, "CommandModel", [](QQmlEngine *engine2, QJSEngine *scriptEngine) -> QObject * {
+        Q_UNUSED(scriptEngine);
+        Q_UNUSED(engine2);
+        return new CommandModel();
+    });
     qmlRegisterType<SortFilterRoomListModel>("org.kde.neochat", 1, 0, "SortFilterRoomListModel");
     qmlRegisterType<DevicesModel>("org.kde.neochat", 1, 0, "DevicesModel");
     qmlRegisterUncreatableType<RoomMessageEvent>("org.kde.neochat", 1, 0, "RoomMessageEvent", "ENUM");
@@ -142,6 +186,7 @@ int main(int argc, char *argv[])
 
     QCommandLineParser parser;
     parser.setApplicationDescription(i18n("Client for the matrix communication protocol"));
+    parser.addPositionalArgument(QStringLiteral("urls"), i18n("Supports appstream: url scheme"));
 
     about.setupCommandLine(&parser);
     parser.process(app);
@@ -157,6 +202,10 @@ int main(int argc, char *argv[])
         return -1;
     }
 
+    if (parser.positionalArguments().length() > 0) {
+        RoomManager::instance().setUrlArgument(parser.positionalArguments()[0]);
+    }
+
 #ifdef HAVE_KDBUSADDONS
     QObject::connect(&service, &KDBusService::activateRequested, &engine, [&engine](const QStringList & /*arguments*/, const QString & /*workingDirectory*/) {
         const auto rootObjects = engine.rootObjects();
@@ -171,7 +220,7 @@ int main(int argc, char *argv[])
     });
     const auto rootObjects = engine.rootObjects();
     for (auto obj : rootObjects) {
-        auto view = qobject_cast<QQuickWindow*>(obj);
+        auto view = qobject_cast<QQuickWindow *>(obj);
         if (view) {
             KConfig dataResource("data", KConfig::SimpleConfig, QStandardPaths::AppDataLocation);
             KConfigGroup windowGroup(&dataResource, "Window");
@@ -181,5 +230,5 @@ int main(int argc, char *argv[])
         }
     }
 #endif
-    return QApplication::exec();
+    return app.exec();
 }
