@@ -1,8 +1,6 @@
-/**
- * SPDX-FileCopyrightText: 2018-2019 Black Hat <bhat@encom.eu.org>
- *
- * SPDX-License-Identifier: GPL-3.0-only
- */
+// SPDX-FileCopyrightText: 2018-2019 Black Hat <bhat@encom.eu.org>
+// SPDX-License-Identifier: GPL-3.0-only
+
 #include "messageeventmodel.h"
 
 #include "neochatconfig.h"
@@ -21,6 +19,7 @@
 #include <QTimeZone>
 
 #include <KLocalizedString>
+#include <KFormat>
 
 #include "utils.h"
 
@@ -36,7 +35,6 @@ QHash<int, QByteArray> MessageEventModel::roleNames() const
     roles[ContentRole] = "content";
     roles[ContentTypeRole] = "contentType";
     roles[HighlightRole] = "isHighlighted";
-    roles[ReadMarkerRole] = "readMarker";
     roles[SpecialMarksRole] = "marks";
     roles[LongOperationRole] = "progressInfo";
     roles[AnnotationRole] = "annotation";
@@ -92,6 +90,7 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
 
     m_currentRoom = room;
     if (room) {
+        m_lastReadEventIndex = QPersistentModelIndex(QModelIndex());
         room->setDisplayed();
         if (m_currentRoom->timelineSize() < 10) {
             room->getPreviousContent(50);
@@ -100,6 +99,39 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
 
         using namespace Quotient;
         connect(m_currentRoom, &Room::aboutToAddNewMessages, this, [=](RoomEventsRange events) {
+            if (NeoChatConfig::self()->showFancyEffects()) {
+                for (auto &event : events) {
+                    RoomMessageEvent *message = dynamic_cast<RoomMessageEvent *>(event.get());
+                    if (message) {
+                        QString planBody = message->plainBody();
+                        // snowflake
+                        const QString snowlakeEmoji = QString::fromUtf8("\xE2\x9D\x84");
+                        if (planBody.contains(snowlakeEmoji)) {
+                            Q_EMIT fancyEffectsReasonFound(QStringLiteral("snowflake"));
+                        }
+                        // fireworks
+                        const QString fireworksEmoji = QString::fromUtf8("\xF0\x9F\x8E\x86");
+                        if (planBody.contains(fireworksEmoji)) {
+                            Q_EMIT fancyEffectsReasonFound(QStringLiteral("fireworks"));
+                        }
+                        // sparkler
+                        const QString sparklerEmoji = QString::fromUtf8("\xF0\x9F\x8E\x87");
+                        if (planBody.contains(sparklerEmoji)) {
+                            Q_EMIT fancyEffectsReasonFound(QStringLiteral("fireworks"));
+                        }
+                        // party pooper
+                        const QString partyEmoji = QString::fromUtf8("\xF0\x9F\x8E\x89");
+                        if (planBody.contains(partyEmoji)) {
+                            Q_EMIT fancyEffectsReasonFound(QStringLiteral("confetti"));
+                        }
+                        // confetti ball
+                        const QString confettiEmoji = QString::fromUtf8("\xF0\x9F\x8E\x8A");
+                        if (planBody.contains(confettiEmoji)) {
+                            Q_EMIT fancyEffectsReasonFound(QStringLiteral("confetti"));
+                        }
+                    }
+                }
+            }
             beginInsertRows({}, timelineBaseIndex(), timelineBaseIndex() + int(events.size()) - 1);
         });
         connect(m_currentRoom, &Room::aboutToAddHistoricalMessages, this, [=](RoomEventsRange events) {
@@ -110,6 +142,10 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
         });
         connect(m_currentRoom, &Room::addedMessages, this, [=](int lowest, int biggest) {
             endInsertRows();
+            if (!m_lastReadEventIndex.isValid()) {
+                // no read marker, so see if we need to create one.
+                moveReadMarker(QString(), m_currentRoom->readMarkerEventId());
+            }
             if (biggest < m_currentRoom->maxTimelineIndex()) {
                 auto rowBelowInserted = m_currentRoom->maxTimelineIndex() - biggest + timelineBaseIndex() - 1;
                 refreshEventRoles(rowBelowInserted, {ShowAuthorRole});
@@ -139,9 +175,6 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
             }
             refreshRow(timelineBaseIndex()); // Refresh the looks
             refreshLastUserEvents(0);
-            if (m_currentRoom->timelineSize() > 1) { // Refresh above
-                refreshEventRoles(timelineBaseIndex() + 1, {ReadMarkerRole});
-            }
             if (timelineBaseIndex() > 0) { // Refresh below, see #312
                 refreshEventRoles(timelineBaseIndex() - 1, {ShowAuthorRole});
             }
@@ -151,10 +184,7 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
             beginRemoveRows({}, i, i);
         });
         connect(m_currentRoom, &Room::pendingEventDiscarded, this, &MessageEventModel::endRemoveRows);
-        connect(m_currentRoom, &Room::readMarkerMoved, this, [this] {
-            refreshEventRoles(std::exchange(lastReadEventId, m_currentRoom->readMarkerEventId()), {ReadMarkerRole});
-            refreshEventRoles(lastReadEventId, {ReadMarkerRole});
-        });
+        connect(m_currentRoom, &Room::readMarkerMoved, this, &MessageEventModel::moveReadMarker);
         connect(m_currentRoom, &Room::replacedEvent, this, [this](const RoomEvent *newEvent) {
             refreshLastUserEvents(refreshEvent(newEvent->id()) - timelineBaseIndex());
         });
@@ -168,10 +198,6 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
         connect(m_currentRoom, &Room::fileTransferCompleted, this, &MessageEventModel::refreshEvent);
         connect(m_currentRoom, &Room::fileTransferFailed, this, &MessageEventModel::refreshEvent);
         connect(m_currentRoom, &Room::fileTransferCancelled, this, &MessageEventModel::refreshEvent);
-        connect(m_currentRoom, &Room::readMarkerForUserMoved, this, [=](User *, const QString &fromEventId, const QString &toEventId) {
-            refreshEventRoles(fromEventId, {UserMarkerRole});
-            refreshEventRoles(toEventId, {UserMarkerRole});
-        });
         connect(m_currentRoom->connection(), &Connection::ignoredUsersListChanged, this, [=] {
             beginResetModel();
             endResetModel();
@@ -204,6 +230,43 @@ void MessageEventModel::refreshEventRoles(int row, const QVector<int> &roles)
     Q_EMIT dataChanged(idx, idx, roles);
 }
 
+void MessageEventModel::moveReadMarker(const QString &fromEventId, const QString &toEventId)
+{
+    const auto timelineIt = m_currentRoom->findInTimeline(toEventId);
+    if (timelineIt == m_currentRoom->timelineEdge()) {
+        return;
+    }
+    int newRow = int(timelineIt - m_currentRoom->messageEvents().rbegin()) + timelineBaseIndex();
+
+    if (!m_lastReadEventIndex.isValid()) {
+        // Not valid index means we don't display any marker yet, in this case
+        // we create the new index and insert the row in case the read marker
+        // need to be displayed.
+        if (newRow > timelineBaseIndex()) {
+            // The user didn't read all the messages yet.
+            beginInsertRows({}, newRow, newRow);
+            m_lastReadEventIndex = QPersistentModelIndex(index(newRow, 0));
+            endInsertRows();
+            return;
+        }
+        // The user read all the messages and we didn't display any read marker yet
+        // => do nothing
+        return;
+    }
+    if (newRow <= timelineBaseIndex()) {
+        // The user read all the messages => remove read marker
+        beginRemoveRows({}, m_lastReadEventIndex.row(), m_lastReadEventIndex.row());
+        m_lastReadEventIndex = QModelIndex();
+        endRemoveRows();
+        return;
+    }
+
+    // The user didn't read all the messages yet but moved the reader marker.
+    beginMoveRows({}, m_lastReadEventIndex.row(), m_lastReadEventIndex.row(), {}, newRow);
+    m_lastReadEventIndex = QPersistentModelIndex(index(newRow, 0));
+    endMoveRows();
+}
+
 int MessageEventModel::refreshEventRoles(const QString &id, const QVector<int> &roles)
 {
     // On 64-bit platforms, difference_type for std containers is long long
@@ -296,9 +359,33 @@ int MessageEventModel::rowCount(const QModelIndex &parent) const
     if (!m_currentRoom || parent.isValid()) {
         return 0;
     }
-    return m_currentRoom->timelineSize();
+
+    const auto firstIt = m_currentRoom->messageEvents().crbegin();
+    if (firstIt != m_currentRoom->messageEvents().crend()) {
+        const auto &firstEvt = **firstIt;
+        return m_currentRoom->timelineSize() + (lastReadEventId != firstEvt.id() ? 1 : 0);
+    } else {
+        return m_currentRoom->timelineSize();
+    }
+
 }
 
+bool MessageEventModel::canFetchMore(const QModelIndex &parent) const
+{
+    Q_UNUSED(parent);
+
+    return m_currentRoom && !m_currentRoom->eventsHistoryJob() && !m_currentRoom->allHistoryLoaded();
+}
+
+void MessageEventModel::fetchMore(const QModelIndex &parent)
+{
+    Q_UNUSED(parent);
+    if (m_currentRoom) {
+        m_currentRoom->getPreviousContent(20);
+    }
+}
+
+
 inline QVariantMap userAtEvent(NeoChatUser *user, NeoChatRoom *room, const RoomEvent &evt)
 {
     Q_UNUSED(evt)
@@ -323,14 +410,32 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
     };
 
     bool isPending = row < timelineBaseIndex();
-    const auto timelineIt = m_currentRoom->messageEvents().crbegin() + std::max(0, row - timelineBaseIndex());
+
+    if (m_lastReadEventIndex.row() == row) {
+        switch(role) {
+        case EventTypeRole:
+            return QStringLiteral("readMarker");
+        case TimeRole:
+            {
+                const QDateTime eventDate = data(index(m_lastReadEventIndex.row() + 1, 0), TimeRole).toDateTime();
+                const KFormat format;
+                return format.formatRelativeDateTime(eventDate, QLocale::ShortFormat);
+            }
+        case SpecialMarksRole:
+            return EventStatus::Hidden;
+        }
+        return {};
+    }
+
+    const auto timelineIt = m_currentRoom->messageEvents().crbegin() + std::max(0, row - timelineBaseIndex() - (m_lastReadEventIndex.isValid() && m_lastReadEventIndex.row() < row ? 1 : 0));
     const auto pendingIt = m_currentRoom->pendingEvents().crbegin() + std::min(row, timelineBaseIndex());
     const auto &evt = isPending ? **pendingIt : **timelineIt;
 
     if (role == Qt::DisplayRole) {
         if (evt.isRedacted()) {
             auto reason = evt.redactedBecause()->reason();
-            return (reason.isEmpty()) ? i18n("<i>[This message was deleted]</i>") : i18n("<i>[This message was deleted: %1]</i>").arg(evt.redactedBecause()->reason());
+            return (reason.isEmpty()) ? i18n("<i>[This message was deleted]</i>")
+                                      : i18n("<i>[This message was deleted: %1]</i>").arg(evt.redactedBecause()->reason());
         }
 
         return m_currentRoom->eventToString(evt, Qt::RichText);
@@ -425,10 +530,6 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
         return m_currentRoom->isEventHighlighted(&evt);
     }
 
-    if (role == ReadMarkerRole) {
-        return evt.id() == lastReadEventId && row > timelineBaseIndex();
-    }
-
     if (role == SpecialMarksRole) {
         if (isPending) {
             return pendingIt->deliveryStatus();
@@ -449,9 +550,6 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
         if (is<RedactionEvent>(evt) || is<ReactionEvent>(evt)) {
             return EventStatus::Hidden;
         }
-        if (evt.isRedacted()) {
-            return EventStatus::Hidden;
-        }
 
         if (evt.isStateEvent() && static_cast<const StateEventBase &>(evt).repeatsState()) {
             return EventStatus::Hidden;
@@ -472,7 +570,8 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
 
     if (role == IsEditedRole) {
         if (auto e = eventCast<const RoomMessageEvent>(&evt)) {
-            return !e->unsignedJson().isEmpty() && e->unsignedJson().contains("m.relations") && e->unsignedJson()["m.relations"].toObject().contains("m.replace");
+            return !e->unsignedJson().isEmpty() && e->unsignedJson().contains("m.relations")
+                && e->unsignedJson()["m.relations"].toObject().contains("m.replace");
         }
         return false;
     }
@@ -526,14 +625,65 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
         };
         const auto &replyEvt = **replyIt;
 
-        return QVariantMap{{"eventId", replyEventId}, {"display", m_currentRoom->eventToString(replyEvt, Qt::RichText)}, {"author", userAtEvent(static_cast<NeoChatUser *>(m_currentRoom->user(replyEvt.senderId())), m_currentRoom, evt)}};
+        QString type;
+        if (auto e = eventCast<const RoomMessageEvent>(&replyEvt)) {
+            switch (e->msgtype()) {
+            case MessageEventType::Emote:
+                type = "emote";
+                break;
+            case MessageEventType::Notice:
+                type = "notice";
+                break;
+            case MessageEventType::Image:
+                type = "image";
+                break;
+            case MessageEventType::Audio:
+                type = "audio";
+                break;
+            case MessageEventType::Video:
+                type = "video";
+                break;
+            default:
+                if (e->hasFileContent()) {
+                    type = "file";
+                    break;
+                }
+                type = "message";
+            }
+
+        } else if (is<const StickerEvent>(replyEvt)) {
+            type = "sticker";
+        } else {
+            type = "other";
+        }
+
+        QVariant content;
+        if (auto e = eventCast<const RoomMessageEvent>(&replyEvt)) {
+            // Cannot use e.contentJson() here because some
+            // EventContent classes inject values into the copy of the
+            // content JSON stored in EventContent::Base
+            content = e->hasFileContent() ? QVariant::fromValue(e->content()->originalJson) : QVariant();
+        };
+
+        if (auto e = eventCast<const StickerEvent>(&replyEvt)) {
+            content = QVariant::fromValue(e->image().originalJson);
+        }
+
+        return QVariantMap{
+            {"eventId", replyEventId},
+            {"display", m_currentRoom->eventToString(replyEvt, Qt::RichText)},
+            {"content", content},
+            {"type", type},
+            {"author", userAtEvent(static_cast<NeoChatUser *>(m_currentRoom->user(replyEvt.senderId())), m_currentRoom, evt)}};
     }
 
     if (role == ShowAuthorRole) {
         for (auto r = row + 1; r < rowCount(); ++r) {
             auto i = index(r);
             if (data(i, SpecialMarksRole) != EventStatus::Hidden) {
-                return data(i, AuthorRole) != data(idx, AuthorRole) || data(i, EventTypeRole) != data(idx, EventTypeRole) || data(idx, TimeRole).toDateTime().msecsTo(data(i, TimeRole).toDateTime()) > 600000;
+                return data(i, AuthorRole) != data(idx, AuthorRole) || data(i, EventTypeRole) != data(idx, EventTypeRole)
+                    || data(idx, TimeRole).toDateTime().msecsTo(data(i, TimeRole).toDateTime()) > 600000
+                    || data(idx, TimeRole).toDateTime().toLocalTime().date().day() != data(i, TimeRole).toDateTime().toLocalTime().date().day();
             }
         }
 
@@ -550,7 +700,7 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
             }
         }
 
-        return true;
+        return false;
     }
 
     if (role == ReactionRole) {
@@ -594,8 +744,83 @@ int MessageEventModel::eventIDToIndex(const QString &eventID) const
 {
     const auto it = m_currentRoom->findInTimeline(eventID);
     if (it == m_currentRoom->timelineEdge()) {
-        qWarning() << "Trying to find inexistent event:" << eventID;
+        //qWarning() << "Trying to find inexistent event:" << eventID;
         return -1;
     }
     return it - m_currentRoom->messageEvents().rbegin() + timelineBaseIndex();
 }
+
+QVariant MessageEventModel::getLastLocalUserMessageEventId()
+{
+    QVariantMap targetMessage;
+    const auto &timelineBottom = m_currentRoom->messageEvents().rbegin();
+
+    // set a cap limit of 35 messages, to prevent loading a lot of messages
+    // in rooms where the user has not sent many messages
+    const auto limit = timelineBottom + std::min(35, m_currentRoom->timelineSize());
+
+    for (auto it = timelineBottom; it != limit; ++it) {
+        auto evt = it->event();
+        auto e = eventCast<const RoomMessageEvent>(evt);
+
+        // check if the current message's sender's id is same as the user's id
+        if ((*it)->senderId() == m_currentRoom->localUser()->id()) {
+            auto content = (*it)->contentJson();
+
+            if (content.contains("m.relates_to")) {
+                // the message has been edited once
+                // so we have to return the id of the related' message instead
+                targetMessage.insert("event_id", content["m.relates_to"].toObject()["event_id"].toString());
+                targetMessage.insert("body", content["formatted_body"].toString());
+                return targetMessage;
+            }
+
+            if (e->msgtype() == MessageEventType::Text) {
+                targetMessage.insert("event_id", (*it)->id());
+                targetMessage.insert("body", content["body"].toString());
+                return targetMessage;
+            }
+        }
+    }
+    return targetMessage;
+}
+
+QVariant MessageEventModel::getLatestMessageFromIndex(const int baseline)
+{
+    QVariantMap replyResponse;
+    const auto &timelineBottom = m_currentRoom->messageEvents().rbegin() + baseline;
+
+    // set a cap limit of baseline + 35 messages, to prevent loading a lot of messages
+    // in rooms where the user has not sent many messages
+    const auto limit = timelineBottom + std::min(baseline + 35, m_currentRoom->timelineSize());
+
+    for (auto it = timelineBottom; it != limit; ++it) {
+        auto evt = it->event();
+        auto e = eventCast<const RoomMessageEvent>(evt);
+
+        auto content = (*it)->contentJson();
+
+        if (content.contains("m.relates_to")) {
+            auto relatedContent = content["m.relates_to"].toObject();
+
+            if (!relatedContent.contains("m.in_reply_to")) {
+                // the message has been edited once
+                // so we have to return the id of the related' message instead
+                replyResponse.insert("event_id", relatedContent["event_id"].toString());
+                replyResponse.insert("event", content["m.formatted_body"].toString());
+                replyResponse.insert("sender_id", QVariant::fromValue(m_currentRoom->getUser((*it)->senderId())));
+                replyResponse.insert("at", -it->index());
+                return replyResponse;
+            }
+        }
+
+        if (e->msgtype() != MessageEventType::Unknown) {
+            replyResponse.insert("event_id", (*it)->id());
+            replyResponse.insert("event", content["body"].toString());
+            replyResponse.insert("sender_id", QVariant::fromValue(m_currentRoom->getUser((*it)->senderId())));
+            replyResponse.insert("at", -it->index());
+            return replyResponse;
+        }
+    }
+    return replyResponse;
+}
