@@ -1,8 +1,5 @@
-/**
- * SPDX-FileCopyrightText: 2018-2019 Black Hat <bhat@encom.eu.org>
- *
- * SPDX-License-Identifier: GPL-3.0-only
- */
+// SPDX-FileCopyrightText: 2018-2019 Black Hat <bhat@encom.eu.org>
+// SPDX-License-Identifier: GPL-3.0-only
 
 #pragma once
 
@@ -27,7 +24,6 @@ public:
         ContentRole,
         ContentTypeRole,
         HighlightRole,
-        ReadMarkerRole,
         SpecialMarksRole,
         LongOperationRole,
         AnnotationRole,
@@ -48,13 +44,6 @@ public:
     };
     Q_ENUM(EventRoles)
 
-    enum BubbleShapes {
-        NoShape = 0,
-        BeginShape,
-        MiddleShape,
-        EndShape,
-    };
-
     explicit MessageEventModel(QObject *parent = nullptr);
     ~MessageEventModel() override;
 
@@ -69,6 +58,8 @@ public:
     [[nodiscard]] QHash<int, QByteArray> roleNames() const override;
 
     Q_INVOKABLE [[nodiscard]] int eventIDToIndex(const QString &eventID) const;
+    Q_INVOKABLE [[nodiscard]] QVariant getLastLocalUserMessageEventId();
+    Q_INVOKABLE [[nodiscard]] QVariant getLatestMessageFromIndex(const int baseline);
 
 private Q_SLOTS:
     int refreshEvent(const QString &eventId);
@@ -77,6 +68,7 @@ private Q_SLOTS:
 private:
     NeoChatRoom *m_currentRoom = nullptr;
     QString lastReadEventId;
+    QPersistentModelIndex m_lastReadEventIndex;
     int rowBelowInserted = -1;
     bool movingEvent = false;
 
@@ -84,10 +76,15 @@ private:
     [[nodiscard]] QDateTime makeMessageTimestamp(const Quotient::Room::rev_iter_t &baseIt) const;
     [[nodiscard]] static QString renderDate(const QDateTime &timestamp);
 
+    bool canFetchMore(const QModelIndex &parent) const override;
+    void fetchMore(const QModelIndex &parent) override;
+
     void refreshLastUserEvents(int baseTimelineRow);
     void refreshEventRoles(int row, const QVector<int> &roles = {});
     int refreshEventRoles(const QString &eventId, const QVector<int> &roles = {});
+    void moveReadMarker(const QString &fromEventId, const QString &toEventId);
 
 Q_SIGNALS:
     void roomChanged();
+    void fancyEffectsReasonFound(const QString &fancyEffect);
 };
