@@ -1,13 +1,10 @@
-/**
- * SPDX-FileCopyrightText: 2018-2020 Black Hat <bhat@encom.eu.org>
- *
- * SPDX-License-Identifier: GPL-3.0-only
- */
+// SPDX-FileCopyrightText: 2018-2020 Black Hat <bhat@encom.eu.org>
+// SPDX-License-Identifier: GPL-3.0-only
+
 #include "neochatroom.h"
 
 #include <cmark.h>
 
-#include <QFileDialog>
 #include <QFileInfo>
 #include <QImageReader>
 #include <QMetaObject>
@@ -27,11 +24,11 @@
 #include "events/roomcanonicalaliasevent.h"
 #include "events/roommessageevent.h"
 #include "events/roompowerlevelsevent.h"
-#include "stickerevent.h"
 #include "events/typingevent.h"
 #include "jobs/downloadfilejob.h"
 #include "neochatconfig.h"
 #include "notificationsmanager.h"
+#include "stickerevent.h"
 #include "user.h"
 #include "utils.h"
 
@@ -46,38 +43,13 @@ NeoChatRoom::NeoChatRoom(Connection *connection, QString roomId, JoinState joinS
         setFileUploadingProgress(0);
         setHasFileUploading(false);
     });
-    connect(this, &NeoChatRoom::notificationCountChanged, this, [this]() {
-        if (messageEvents().size() == 0) {
-            return;
-        }
-        const RoomEvent *lastEvent = messageEvents().rbegin()->get();
-        if (lastEvent->originTimestamp() < QDateTime::currentDateTime().addSecs(-60)) {
-            return;
-        }
-        if (lastEvent->isStateEvent()) {
-            return;
-        }
-        User *sender = user(lastEvent->senderId());
-        if (sender == localUser()) {
-            return;
-        }
-
-        QImage avatar_image;
-        if (!sender->avatarUrl(this).isEmpty()) {
-            avatar_image = sender->avatar(128, this);
-        } else {
-            avatar_image = this->avatar(128);
-        }
-
-        NotificationsManager::instance().postNotification(this, displayName(), sender->displayname(this), eventToString(*lastEvent), avatar_image);
-    });
 
     connect(this, &Room::aboutToAddHistoricalMessages, this, &NeoChatRoom::readMarkerLoadedChanged);
 
     connect(this, &Quotient::Room::eventsHistoryJobChanged, this, &NeoChatRoom::lastActiveTimeChanged);
 
     connect(this, &Room::joinStateChanged, this, [=](JoinState oldState, JoinState newState) {
-        if(oldState == JoinState::Invite && newState != JoinState::Invite) {
+        if (oldState == JoinState::Invite && newState != JoinState::Invite) {
             Q_EMIT isInviteChanged();
         }
     });
@@ -126,7 +98,7 @@ QVariantList NeoChatRoom::getUsersTyping() const
     users.removeAll(localUser());
     QVariantList userVariants;
     for (User *user : users) {
-        userVariants.append(QVariantMap {
+        userVariants.append(QVariantMap{
             {"id", user->id()},
             {"avatarMediaId", user->avatarMediaId(this)},
             {"displayName", user->displayname(this)},
@@ -153,7 +125,8 @@ const RoomMessageEvent *NeoChatRoom::lastEvent(bool ignoreStateEvent) const
             continue;
         }
 
-        if (event->isStateEvent() && (ignoreStateEvent || !NeoChatConfig::self()->showLeaveJoinEvent() || static_cast<const StateEventBase &>(*event).repeatsState())) {
+        if (event->isStateEvent()
+            && (ignoreStateEvent || !NeoChatConfig::self()->showLeaveJoinEvent() || static_cast<const StateEventBase &>(*event).repeatsState())) {
             continue;
         }
 
@@ -277,12 +250,10 @@ QVariantList NeoChatRoom::getUsers(const QString &keyword) const
     for (const auto u : userList) {
         if (u->displayname(this).contains(keyword, Qt::CaseInsensitive)) {
             NeoChatUser user(u->id(), u->connection());
-            QVariantMap userVariant {
-                { QStringLiteral("id"), user.id() },
-                { QStringLiteral("displayName"), user.displayname(this) },
-                { QStringLiteral("avatarMediaId"), user.avatarMediaId(this) },
-                { QStringLiteral("color"), user.color() }
-            };
+            QVariantMap userVariant{{QStringLiteral("id"), user.id()},
+                                    {QStringLiteral("displayName"), user.displayname(this)},
+                                    {QStringLiteral("avatarMediaId"), user.avatarMediaId(this)},
+                                    {QStringLiteral("color"), user.color()}};
 
             matchedList.append(QVariant::fromValue(userVariant));
         }
@@ -291,6 +262,17 @@ QVariantList NeoChatRoom::getUsers(const QString &keyword) const
     return matchedList;
 }
 
+QVariantMap NeoChatRoom::getUser(const QString& userID) const
+{
+    NeoChatUser user(userID, connection());
+    return QVariantMap {
+        { QStringLiteral("id"), user.id() },
+        { QStringLiteral("displayName"), user.displayname(this) },
+        { QStringLiteral("avatarMediaId"), user.avatarMediaId(this) },
+        { QStringLiteral("color"), user.color() }
+    };
+}
+
 QUrl NeoChatRoom::urlToMxcUrl(const QUrl &mxcUrl)
 {
     return DownloadFileJob::makeRequestUrl(connection()->homeserver(), mxcUrl);
@@ -382,14 +364,12 @@ QString NeoChatRoom::eventToString(const RoomEvent &evt, Qt::TextFormat format,
                 }
                 Q_FALLTHROUGH();
             case MembershipType::Join: {
-                QString text {};
+                QString text{};
                 // Part 1: invites and joins
                 if (e.repeatsState()) {
                     text = i18n("joined the room (repeated)");
                 } else if (e.changesMembership()) {
-                    text = e.membership() == MembershipType::Invite
-                            ? i18n("invited %1 to the room", subjectName)
-                            : i18n("joined the room");
+                    text = e.membership() == MembershipType::Invite ? i18n("invited %1 to the room", subjectName) : i18n("joined the room");
                 }
                 if (!text.isEmpty()) {
                     if (!e.reason().isEmpty()) {
@@ -400,9 +380,9 @@ QString NeoChatRoom::eventToString(const RoomEvent &evt, Qt::TextFormat format,
                 // Part 2: profile changes of joined members
                 if (e.isRename() && NeoChatConfig::self()->showRename()) {
                     if (!e.displayName().isEmpty()) {
-                        text = i18n("cleared their display name");
+                        text = i18nc("their refers to a singular user", "cleared their display name");
                     } else {
-                        text = i18n("changed their display name to %1", e.displayName().toHtmlEscaped());
+                        text = i18nc("their refers to a singular user", "changed their display name to %1", e.displayName().toHtmlEscaped());
                     }
                 }
                 if (e.isAvatarUpdate() && NeoChatConfig::self()->showAvatarUpdate()) {
@@ -410,11 +390,15 @@ QString NeoChatRoom::eventToString(const RoomEvent &evt, Qt::TextFormat format,
                         text += i18n(" and ");
                     }
                     if (e.avatarUrl().isEmpty()) {
-                        text += i18n("cleared their avatar");
+                        text += i18nc("their refers to a singular user", "cleared their avatar");
+#ifdef QUOTIENT_07
+                    } else if (!e.prevContent()->avatarUrl) {
+#else
                     } else if (e.prevContent()->avatarUrl.isEmpty()) {
+#endif
                         text += i18n("set an avatar");
                     } else {
-                        text += i18n("updated their avatar");
+                        text += i18nc("their refers to a singular user", "updated their avatar");
                     }
                 }
                 return text;
@@ -427,9 +411,12 @@ QString NeoChatRoom::eventToString(const RoomEvent &evt, Qt::TextFormat format,
                 if (e.prevContent() && e.prevContent()->membership == MembershipType::Ban) {
                     return (e.senderId() != e.userId()) ? i18n("unbanned %1", subjectName) : i18n("self-unbanned");
                 }
-                return (e.senderId() != e.userId()) ? i18n("has put %1 out of the room: %2", subjectName, e.contentJson()["reason"_ls].toString().toHtmlEscaped()) : i18n("left the room");
+                return (e.senderId() != e.userId())
+                    ? i18n("has put %1 out of the room: %2", subjectName, e.contentJson()["reason"_ls].toString().toHtmlEscaped())
+                    : i18n("left the room");
             case MembershipType::Ban:
-                return (e.senderId() != e.userId()) ? i18n("banned %1 from the room: %2", subjectName, e.contentJson()["reason"_ls].toString().toHtmlEscaped()) : i18n("self-banned from the room");
+                return (e.senderId() != e.userId()) ? i18n("banned %1 from the room: %2", subjectName, e.contentJson()["reason"_ls].toString().toHtmlEscaped())
+                                                    : i18n("self-banned from the room");
             case MembershipType::Knock:
                 return i18n("knocked");
             default:;
@@ -452,11 +439,14 @@ QString NeoChatRoom::eventToString(const RoomEvent &evt, Qt::TextFormat format,
             return i18n("activated End-to-End Encryption");
         },
         [](const RoomCreateEvent &e) {
-            return e.isUpgrade() ? i18n("upgraded the room to version %1", e.version().isEmpty() ? "1" : e.version().toHtmlEscaped()) : i18n("created the room, version %1", e.version().isEmpty() ? "1" : e.version().toHtmlEscaped());
+            return e.isUpgrade() ? i18n("upgraded the room to version %1", e.version().isEmpty() ? "1" : e.version().toHtmlEscaped())
+                                 : i18n("created the room, version %1", e.version().isEmpty() ? "1" : e.version().toHtmlEscaped());
         },
         [](const StateEventBase &e) {
             // A small hack for state events from TWIM bot
-            return e.stateKey() == "twim" ? i18n("updated the database") : e.stateKey().isEmpty() ? i18n("updated %1 state", e.matrixType()) : i18n("updated %1 state for %2", e.matrixType(), e.stateKey().toHtmlEscaped());
+            return e.stateKey() == "twim" ? i18n("updated the database")
+                : e.stateKey().isEmpty()  ? i18n("updated %1 state", e.matrixType())
+                                          : i18n("updated %1 state for %2", e.matrixType(), e.stateKey().toHtmlEscaped());
         },
         i18n("Unknown event"));
 }
@@ -464,7 +454,11 @@ QString NeoChatRoom::eventToString(const RoomEvent &evt, Qt::TextFormat format,
 void NeoChatRoom::changeAvatar(const QUrl &localFile)
 {
     const auto job = connection()->uploadFile(localFile.toLocalFile());
+#ifdef QUOTIENT_07
+    if(isJobPending(job)) {
+#else
     if (isJobRunning(job)) {
+#endif
         connect(job, &BaseJob::success, this, [this, job] {
             connection()->callApi<SetRoomStateWithKeyJob>(id(), "m.room.avatar", localUser()->id(), QJsonObject{{"url", job->contentUri()}});
         });
@@ -556,29 +550,15 @@ void NeoChatRoom::postHtmlMessage(const QString &text, const QString &html, Mess
         isReply = false;
     }
 
-
     if (isEdit) {
-        QJsonObject json {
+        QJsonObject json{
             {"type", "m.room.message"},
             {"msgtype", msgTypeToString(type)},
             {"body", "* " + text},
             {"format", "org.matrix.custom.html"},
             {"formatted_body", html},
-            {"m.new_content",
-              QJsonObject {
-                {"body", text},
-                {"msgtype", msgTypeToString(type)},
-                {"format", "org.matrix.custom.html"},
-                {"formatted_body", html}
-              }
-            },
-            {"m.relates_to",
-                QJsonObject {
-                    {"rel_type", "m.replace"},
-                    {"event_id", relateToEventId}
-                }
-            }
-        };
+            {"m.new_content", QJsonObject{{"body", text}, {"msgtype", msgTypeToString(type)}, {"format", "org.matrix.custom.html"}, {"formatted_body", html}}},
+            {"m.relates_to", QJsonObject{{"rel_type", "m.replace"}, {"event_id", relateToEventId}}}};
 
         postJson("m.room.message", json);
         return;
@@ -704,3 +684,8 @@ bool NeoChatRoom::isInvite() const
 {
     return joinState() == JoinState::Invite;
 }
+
+bool NeoChatRoom::isUserBanned(const QString &user) const
+{
+    return getCurrentState<RoomMemberEvent>(user)->membership() == MembershipType::Ban;
+}
