@@ -1,8 +1,5 @@
-/**
- * SPDX-FileCopyrightText: 2020 Tobias Fella <fella@posteo.de>
- *
- * SPDX-License-Identifier: GPL-2.0-or-later
- */
+// SPDX-FileCopyrightText: 2020 Tobias Fella <fella@posteo.de>
+// SPDX-License-Identifier: GPL-2.0-or-later
 
 #pragma once
 
@@ -22,7 +19,8 @@ class NotificationsManager : public QObject
 public:
     static NotificationsManager &instance();
 
-    Q_INVOKABLE void postNotification(NeoChatRoom *room, const QString &roomName, const QString &sender, const QString &text, const QImage &icon);
+    Q_INVOKABLE void
+    postNotification(NeoChatRoom *room, const QString &roomName, const QString &sender, const QString &text, const QImage &icon, const QString &replyEventId);
 
 private:
     NotificationsManager(QObject *parent = nullptr);
