@@ -1,8 +1,6 @@
-/**
- * SPDX-FileCopyrightText: 2018-2019 Black Hat <bhat@encom.eu.org>
- *
- * SPDX-License-Identifier: GPL-3.0-only
- */
+// SPDX-FileCopyrightText: 2018-2019 Black Hat <bhat@encom.eu.org>
+// SPDX-License-Identifier: GPL-3.0-only
+
 #include "roomlistmodel.h"
 
 #include "neochatconfig.h"
@@ -16,19 +14,22 @@
 #include <QDebug>
 #ifndef Q_OS_ANDROID
 #include <QDBusConnection>
-#include <QDBusMessage>
 #include <QDBusInterface>
+#include <QDBusMessage>
 #endif
 #include <QStandardPaths>
 
 #include <KLocalizedString>
 #include <utility>
 
+#include "csapi/notifications.h"
+#include "notificationsmanager.h"
+#include "roommanager.h"
+
 #ifndef Q_OS_ANDROID
-bool useUnityCounter() {
-    static const auto Result = QDBusInterface(
-        "com.canonical.Unity",
-        "/").isValid();
+bool useUnityCounter()
+{
+    static const auto Result = QDBusInterface("com.canonical.Unity", "/").isValid();
 
     return Result;
 }
@@ -58,15 +59,9 @@ RoomListModel::RoomListModel(QObject *parent)
                 dbusUnityProperties["count-visible"] = false;
             }
 
-            auto signal = QDBusMessage::createSignal(
-                "/com/canonical/unity/launcherentry/neochat",
-                "com.canonical.Unity.LauncherEntry",
-                "Update");
+            auto signal = QDBusMessage::createSignal("/com/canonical/unity/launcherentry/neochat", "com.canonical.Unity.LauncherEntry", "Update");
 
-            signal.setArguments({
-                launcherUrl,
-                dbusUnityProperties
-            });
+            signal.setArguments({launcherUrl, dbusUnityProperties});
 
             QDBusConnection::sessionBus().send(signal);
         }
@@ -120,6 +115,7 @@ void RoomListModel::setConnection(Connection *connection)
 
     doResetModel();
 
+    handleNotifications();
     Q_EMIT connectionChanged();
 }
 
@@ -175,28 +171,7 @@ void RoomListModel::connectRoomSignals(NeoChatRoom *room)
     connect(room, &Room::addedMessages, this, [=] {
         refresh(room, {LastEventRole});
     });
-    connect(room, &Room::notificationCountChanged, this, [=] {
-        if (room->notificationCount() == 0) {
-            return;
-        }
-        if (room->timelineSize() == 0) {
-            return;
-        }
-        auto *lastEvent = room->lastEvent();
-
-        if (!lastEvent) {
-            return;
-        }
-
-        if (lastEvent->isStateEvent()) {
-            return;
-        }
-        User *sender = room->user(lastEvent->senderId());
-        if (sender == room->localUser()) {
-            return;
-        }
-        Q_EMIT newMessage(room->id(), lastEvent->id(), room->displayName(), sender->displayname(), room->eventToString(*lastEvent), room->avatar(128));
-    });
+    connect(room, &Room::notificationCountChanged, this, &RoomListModel::handleNotifications);
     connect(room, &Room::highlightCountChanged, this, [=] {
         if (room->highlightCount() == 0) {
             return;
@@ -222,6 +197,53 @@ void RoomListModel::connectRoomSignals(NeoChatRoom *room)
     connect(room, &Room::notificationCountChanged, this, &RoomListModel::refreshNotificationCount);
 }
 
+void RoomListModel::handleNotifications()
+{
+    static bool initial = true;
+    static QStringList oldNotifications;
+    auto job = m_connection->callApi<GetNotificationsJob>();
+
+    connect(job, &BaseJob::success, this, [=]() {
+        const auto notifications = job->jsonData()["notifications"].toArray();
+        if (initial) {
+            initial = false;
+            for (const auto &n : notifications) {
+                oldNotifications += n.toObject()["event"].toObject()["event_id"].toString();
+            }
+            return;
+        }
+        for (const auto &n : notifications) {
+            const auto notification = n.toObject();
+            if (notification["read"].toBool()) {
+                oldNotifications.removeOne(notification["event"].toObject()["event_id"].toString());
+                continue;
+            }
+            if (oldNotifications.contains(notification["event"].toObject()["event_id"].toString())) {
+                continue;
+            }
+            oldNotifications += notification["event"].toObject()["event_id"].toString();
+            auto room = m_connection->room(notification["room_id"].toString());
+            if (room) {
+                // The room might have been deleted (for example rejected invitation).
+                auto sender = room->user(notification["event"].toObject()["sender"].toString());
+
+                QImage avatar_image;
+                if (!sender->avatarUrl(room).isEmpty()) {
+                    avatar_image = sender->avatar(128, room);
+                } else {
+                    avatar_image = room->avatar(128);
+                }
+                NotificationsManager::instance().postNotification(dynamic_cast<NeoChatRoom *>(room),
+                                                                  room->displayName(),
+                                                                  sender->displayname(room),
+                                                                  notification["event"].toObject()["content"].toObject()["body"].toString(),
+                                                                  avatar_image,
+                                                                  notification["event"].toObject()["event_id"].toString());
+            }
+        }
+    });
+}
+
 void RoomListModel::refreshNotificationCount()
 {
     int count = 0;
@@ -267,6 +289,7 @@ void RoomListModel::updateRoom(Room *room, Room *prev)
     } else {
         beginInsertRows(QModelIndex(), m_rooms.count(), m_rooms.count());
         doAddRoom(newRoom);
+        RoomManager::instance().enterRoom(qobject_cast<NeoChatRoom *>(newRoom));
         endInsertRows();
     }
 }
@@ -323,12 +346,12 @@ QVariant RoomListModel::data(const QModelIndex &index, int role) const
         if (room->isFavourite()) {
             return RoomType::Favorite;
         }
-        if (room->isDirectChat()) {
-            return RoomType::Direct;
-        }
         if (room->isLowPriority()) {
             return RoomType::Deprioritized;
         }
+        if (room->isDirectChat()) {
+            return RoomType::Direct;
+        }
         return RoomType::Normal;
     }
     if (role == UnreadCountRole) {
@@ -432,8 +455,8 @@ bool RoomListModel::categoryVisible(int category) const
 
 NeoChatRoom *RoomListModel::roomByAliasOrId(const QString &aliasOrId)
 {
-    for(const auto &room : m_rooms) {
-        if(room->aliases().contains(aliasOrId) || room->id() == aliasOrId) {
+    for (const auto &room : m_rooms) {
+        if (room->aliases().contains(aliasOrId) || room->id() == aliasOrId) {
             return room;
         }
     }
